'''Game main module.

Contains the entry point used by the run_game.py script.

Feel free to put all your game code here, or in other modules in this "lib"
directory.
'''

import os, sys, optparse, threading

import pygame
from pygame.locals import *

import config
from ResourceManager import *
from Sprite import *
from Scene import *

try:
	import psyco
	psyco.profile()
except ImportError:
	print "Warning: Unable to import psyco. Disabling..."

def main():

	parser = optparse.OptionParser()
	parser.add_option("-w", "--windowed",
		action = "store_false",
		dest = "fullscreen",
		help = "start the game in windowed mode. The default is fullscreen mode"
	)

	parser.add_option("-m", "--no-music",
		action = "store_false",
		dest = "music",
		help = "Disables music during the game."
	)

	parser.set_defaults(
		fullscreen = True,
		music = True
	)

	options, args = parser.parse_args()

	config.enable_music = options.music
	
	pygame.init()
	
	if options.fullscreen:
		display = pygame.display.set_mode((config.SCREEN_WIDTH, config.SCREEN_HEIGHT), FULLSCREEN | DOUBLEBUF | HWSURFACE, 32)
	else:
		display = pygame.display.set_mode((config.SCREEN_WIDTH, config.SCREEN_HEIGHT), 0, 32)
	
	pygame.display.set_caption(config.WINDOW_TITLE)

	SpriteManager(config.SPRITELIST_FILE)
	ImageManager()
	SoundManager()
	FontManager()

	scene = LoadingScene(display)
	scene.run()

	if config.enable_music:
		config.music = config.sound_manager["141th_street_blues.ogg"]
		config.music.play(-1)

	result = 0

	while result != 4:
		scene = MainMenuScene(display)
		result = scene.run()

		if result == 0:
			# Story mode easy
			scene = GameScene(display, FileGameFlow(config.LEVEL_EASY))
	
		elif result == 1:
			# Story mode - hard
			scene = GameScene(display, FileGameFlow(config.LEVEL_HARD))

		elif result == 2:
			# Endless mode
			scene = GameScene(display, EndlessGameFlow())
	
		elif result == 3:
			# Endless mode
			scene = HowtoScene(display)

		elif result == -1:
			return

		if result != 4:
			try:
				scene.run()
			finally:
				if isinstance(scene, GameScene) and scene.mouseSampler.isAlive():
					scene.mouseSampler.stop()
					scene.mouseSampler.join()

